@@ -151,6 +151,7 @@ class BuilddBase(Base):
         self,
         *,
         alias: BuilddBaseAlias,
+        compatibility_tag: Optional[str] = None,
         environment: Optional[Dict[str, Optional[str]]] = None,
         hostname: str = "craft-buildd-instance",
         snaps: Optional[List[Snap]] = None,
@@ -163,6 +164,9 @@ class BuilddBase(Base):
         else:
             self.environment = environment
 
+        if compatibility_tag:
+            self.compatibility_tag = compatibility_tag
+
         self._set_hostname(hostname)
         self.snaps = snaps
         self.packages = packages
@@ -181,25 +185,22 @@ class BuilddBase(Base):
         :raises BaseConfigurationError: if the hostname contains no
           alphanumeric characters
         """
+        # truncate to 63 characters
+        truncated_name = hostname[:63]
+
         # remove anything that is not an alphanumeric character or hyphen
-        name_with_valid_chars = re.sub(r"[^\w-]", "", hostname)
+        name_with_valid_chars = re.sub(r"[^\w-]", "", truncated_name)
 
         # trim hyphens from the beginning and end
-        trimmed_name = re.compile(r"^[-]*(?P<valid_name>.*?)[-]*$").search(
-            name_with_valid_chars
-        )
-        if not trimmed_name or trimmed_name.group("valid_name") == "":
+        valid_name = name_with_valid_chars.strip("-")
+        if not valid_name:
             raise BaseConfigurationError(
                 brief=f"failed to create base with hostname {hostname!r}.",
                 details="hostname must contain at least one alphanumeric character",
             )
-        valid_name = trimmed_name.group("valid_name")
 
-        # truncate to 63 characters
-        truncated_name = valid_name[:63]
-
-        logger.debug("Using hostname %r", truncated_name)
-        self.hostname = truncated_name
+        logger.debug("Using hostname %r", valid_name)
+        self.hostname = valid_name
 
     def _ensure_instance_config_compatible(
         self, *, executor: Executor, deadline: Optional[float]
@@ -235,6 +236,9 @@ class BuilddBase(Base):
                     f"{self.compatibility_tag!r}, found {config.compatibility_tag!r}"
                 )
             )
+        logger.debug(
+            "Instance is compatible with compatibility tag %r", config.compatibility_tag
+        )
 
     def _ensure_os_compatible(
         self, *, executor: Executor, deadline: Optional[float]
@@ -346,6 +350,7 @@ class BuilddBase(Base):
         )
         self._setup_apt(executor=executor, deadline=deadline)
         self._setup_snapd(executor=executor, deadline=deadline)
+        self._setup_snapd_proxy(executor=executor, deadline=deadline)
         self._install_snaps(executor=executor, deadline=deadline)
 
     def warmup(
@@ -386,6 +391,7 @@ class BuilddBase(Base):
         self._setup_wait_for_network(
             executor=executor, deadline=deadline, retry_wait=retry_wait
         )
+        self._setup_snapd_proxy(executor=executor, deadline=deadline)
         self._install_snaps(executor=executor, deadline=deadline)
 
     def _disable_automatic_apt(
@@ -722,6 +728,22 @@ class BuilddBase(Base):
                 check=True,
             )
 
+            # This file is created by launchpad-buildd to stop snapd from
+            # using the snap store's CDN when running in Canonical's
+            # production build farm, since internet access restrictions may
+            # prevent it from doing so but will allow the non-CDN storage
+            # endpoint.  If this is in place, then we need to propagate it
+            # to containers we create.
+            no_cdn = pathlib.Path("/etc/systemd/system/snapd.service.d/no-cdn.conf")
+            if no_cdn.exists():
+                _check_deadline(deadline)
+                executor.execute_run(
+                    ["mkdir", "-p", no_cdn.parent.as_posix()], check=True
+                )
+
+                _check_deadline(deadline)
+                executor.push_file(source=no_cdn, destination=no_cdn)
+
             _check_deadline(deadline)
             executor.execute_run(
                 ["apt-get", "install", "-y", "snapd"],
@@ -751,12 +773,44 @@ class BuilddBase(Base):
                 capture_output=True,
                 check=True,
             )
+
         except subprocess.CalledProcessError as error:
             raise BaseConfigurationError(
                 brief="Failed to setup snapd.",
                 details=errors.details_from_called_process_error(error),
             ) from error
 
+    def _setup_snapd_proxy(
+        self, *, executor: Executor, deadline: Optional[float] = None
+    ) -> None:
+        """Configure the snapd proxy.
+
+        :param executor: Executor for target container.
+        :param deadline: Optional time.time() deadline.
+        """
+        try:
+            _check_deadline(deadline)
+            http_proxy = self.environment.get("http_proxy")
+            if http_proxy:
+                command = ["snap", "set", "system", f"proxy.http={http_proxy}"]
+            else:
+                command = ["snap", "unset", "system", "proxy.http"]
+            executor.execute_run(command, capture_output=True, check=True)
+
+            _check_deadline(deadline)
+            https_proxy = self.environment.get("https_proxy")
+            if https_proxy:
+                command = ["snap", "set", "system", f"proxy.https={https_proxy}"]
+            else:
+                command = ["snap", "unset", "system", "proxy.https"]
+            executor.execute_run(command, capture_output=True, check=True)
+
+        except subprocess.CalledProcessError as error:
+            raise BaseConfigurationError(
+                brief="Failed to set the snapd proxy.",
+                details=errors.details_from_called_process_error(error),
+            ) from error
+
     def _setup_wait_for_network(
         self,
         *,
