@@ -39,6 +39,9 @@ class LXDInstallationError(LXDError):
         details: Optional[str] = None,
     ) -> None:
         brief = f"Failed to install LXD: {reason}."
-        resolution = "Please visit https://linuxcontainers.org/lxd/getting-started-cli/ for instructions on installing LXD for your operating system."
+        resolution = (
+            "Please visit https://linuxcontainers.org/lxd/getting-started-cli/"
+            " for instructions on installing LXD for your operating system."
+        )
 
         super().__init__(brief=brief, details=details, resolution=resolution)
