@@ -1,5 +1,5 @@
 #
-# Copyright 2021 Canonical Ltd.
+# Copyright 2021-2022 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -17,10 +17,12 @@
 
 """LXD Instance Executor."""
 
+import hashlib
 import io
 import logging
 import os
 import pathlib
+import re
 import shutil
 import subprocess
 import tempfile
@@ -48,6 +50,17 @@ class LXDInstance(Executor):
         remote: str = "local",
         lxc: Optional[LXC] = None,
     ):
+        """Create an LXD executor.
+
+        To comply with LXD naming conventions, the supplied name is converted to a
+        LXD-compatible name before creating the instance.
+
+        :param name: Unique name of the lxd instance
+        :param default_command_environment: command environment
+        :param project: name of lxd project
+        :param remote: name of lxd remote
+        :param lxc: LXC instance object
+        """
         super().__init__()
 
         if default_command_environment is not None:
@@ -56,6 +69,7 @@ class LXDInstance(Executor):
             self.default_command_environment = {}
 
         self.name = name
+        self._set_instance_name()
         self.project = project
         self.remote = remote
 
@@ -64,6 +78,57 @@ class LXDInstance(Executor):
         else:
             self.lxc = lxc
 
+    def _set_instance_name(self) -> None:
+        """Convert a name to a LXD-compatible name.
+
+        LXD naming convention:
+        - between 1 and 63 characters long
+        - made up exclusively of letters, numbers, and hyphens from the ASCII table
+        - not begin with a digit or a hyphen
+        - not end with a hyphen
+
+        To create a LXD-compatible name, invalid characters are removed, the name is
+        truncated to 40 characters, then a hash is appended:
+        <truncated-name>-<hash-of-name>
+        └     1 - 40   ┘1└     20     ┘
+
+        :param name: name of instance
+        :raises LXDError: if name contains no alphanumeric characters
+        """
+        # remove anything that is not an alphanumeric characters or hyphen
+        name_with_valid_chars = re.sub(r"[^\w-]", "", self.name)
+        if name_with_valid_chars == "":
+            raise LXDError(
+                brief=f"failed to create LXD instance with name {self.name!r}.",
+                details="name must contain at least one alphanumeric character",
+            )
+
+        # trim digits and hyphens from the beginning and hyphens from the end
+        trimmed_name = re.compile(r"^[0-9-]*(?P<valid_name>.*?)[-]*$").search(
+            name_with_valid_chars
+        )
+        if not trimmed_name or trimmed_name.group("valid_name") == "":
+            raise LXDError(
+                brief=f"failed to create LXD instance with name {self.name!r}.",
+                details="name must contain at least one alphanumeric character",
+            )
+        valid_name = trimmed_name.group("valid_name")
+
+        # if the original name meets LXD's naming convention, then use the original name
+        if self.name == valid_name and len(self.name) <= 63:
+            instance_name = self.name
+
+        # else, continue converting the name
+        else:
+            # truncate to 40 characters
+            truncated_name = valid_name[:40]
+            # hash the entire name, not the truncated name
+            hashed_name = hashlib.sha1(self.name.encode()).hexdigest()[:20]
+            instance_name = f"{truncated_name}-{hashed_name}"
+
+        self.instance_name = instance_name
+        logger.debug("Set LXD instance name to %r", instance_name)
+
     def _finalize_lxc_command(
         self,
         command: List[str],
@@ -117,7 +182,7 @@ class LXDInstance(Executor):
 
         temp_path = pathlib.Path(temp_file.name)
         self.lxc.file_push(
-            instance_name=self.name,
+            instance_name=self.instance_name,
             source=temp_path,
             destination=destination,
             mode=file_mode,
@@ -135,7 +200,10 @@ class LXDInstance(Executor):
             )
         except subprocess.CalledProcessError as error:
             raise LXDError(
-                brief=f"Failed to create file {destination.as_posix()!r} in instance {self.name!r}.",
+                brief=(
+                    f"Failed to create file {destination.as_posix()!r}"
+                    f" in instance {self.instance_name!r}."
+                ),
                 details=errors.details_from_called_process_error(error),
             ) from error
 
@@ -149,7 +217,7 @@ class LXDInstance(Executor):
         :raises LXDError: On unexpected error.
         """
         return self.lxc.delete(
-            instance_name=self.name,
+            instance_name=self.instance_name,
             project=self.project,
             remote=self.remote,
             force=force,
@@ -181,7 +249,7 @@ class LXDInstance(Executor):
             cwd_path = cwd.as_posix()
 
         return self.lxc.exec(
-            instance_name=self.name,
+            instance_name=self.instance_name,
             command=self._finalize_lxc_command(command=command, env=env),
             project=self.project,
             remote=self.remote,
@@ -219,7 +287,7 @@ class LXDInstance(Executor):
             cwd_path = cwd.as_posix()
 
         return self.lxc.exec(
-            instance_name=self.name,
+            instance_name=self.instance_name,
             command=self._finalize_lxc_command(command=command, env=env),
             project=self.project,
             remote=self.remote,
@@ -240,7 +308,7 @@ class LXDInstance(Executor):
     def _get_disk_devices(self) -> Dict[str, Any]:
         """Query instance and return dictionary of disk devices."""
         devices = self.lxc.config_device_show(
-            instance_name=self.name, project=self.project, remote=self.remote
+            instance_name=self.instance_name, project=self.project, remote=self.remote
         )
 
         disks = {}
@@ -268,7 +336,7 @@ class LXDInstance(Executor):
         instances = self.lxc.list(project=self.project, remote=self.remote)
 
         for instance in instances:
-            if instance["name"] == self.name:
+            if instance["name"] == self.instance_name:
                 return instance
 
         return None
@@ -302,7 +370,7 @@ class LXDInstance(Executor):
         """
         state = self._get_state()
         if state is None:
-            raise LXDError(brief=f"Instance {self.name!r} does not exist.")
+            raise LXDError(brief=f"Instance {self.instance_name!r} does not exist.")
 
         return state.get("status") == "Running"
 
@@ -320,7 +388,8 @@ class LXDInstance(Executor):
         :param image: Image name to launch.
         :param image_remote: Image remote name.
         :param map_user_id: Whether id mapping should be used.
-        :param uid: If ``map_user_id`` is True, the host user ID to map to instance root.
+        :param uid: If ``map_user_id`` is True,
+                    the host user ID to map to instance root.
         :param ephemeral: Flag to enable ephemeral instance.
 
         :raises LXDError: On unexpected error.
@@ -338,7 +407,7 @@ class LXDInstance(Executor):
         self.lxc.launch(
             config_keys=config_keys,
             ephemeral=ephemeral,
-            instance_name=self.name,
+            instance_name=self.instance_name,
             image=image,
             image_remote=image_remote,
             project=self.project,
@@ -370,7 +439,7 @@ class LXDInstance(Executor):
             device_name = "disk-" + target.as_posix()
 
         self.lxc.config_device_add_disk(
-            instance_name=self.name,
+            instance_name=self.instance_name,
             source=host_source,
             path=target,
             device=device_name,
@@ -413,7 +482,7 @@ class LXDInstance(Executor):
             raise FileNotFoundError(f"Directory not found: {str(destination.parent)!r}")
 
         self.lxc.file_pull(
-            instance_name=self.name,
+            instance_name=self.instance_name,
             source=source,
             destination=destination,
             project=self.project,
@@ -445,7 +514,7 @@ class LXDInstance(Executor):
         # Copy into target with uid/gid 0, rather than copying the IDs from the
         # host file.
         self.lxc.file_push(
-            instance_name=self.name,
+            instance_name=self.instance_name,
             source=source,
             destination=destination,
             project=self.project,
@@ -460,7 +529,7 @@ class LXDInstance(Executor):
         :raises LXDError: on unexpected error.
         """
         self.lxc.start(
-            instance_name=self.name, project=self.project, remote=self.remote
+            instance_name=self.instance_name, project=self.project, remote=self.remote
         )
 
     def stop(self) -> None:
@@ -468,7 +537,9 @@ class LXDInstance(Executor):
 
         :raises LXDError: on unexpected error.
         """
-        self.lxc.stop(instance_name=self.name, project=self.project, remote=self.remote)
+        self.lxc.stop(
+            instance_name=self.instance_name, project=self.project, remote=self.remote
+        )
 
     def supports_mount(self) -> bool:
         """Check if instance supports mounting from host.
@@ -490,7 +561,7 @@ class LXDInstance(Executor):
         for name, config in disks.items():
             if config["path"] == target.as_posix():
                 self.lxc.config_device_remove(
-                    instance_name=self.name,
+                    instance_name=self.instance_name,
                     device=name,
                     project=self.project,
                     remote=self.remote,
@@ -499,7 +570,10 @@ class LXDInstance(Executor):
 
         if not unmounted:
             raise LXDError(
-                brief=f"Failed to unmount {target.as_posix()!r} in instance {self.name!r} - no such disk.",
+                brief=(
+                    f"Failed to unmount {target.as_posix()!r}"
+                    f" in instance {self.instance_name!r} - no such disk."
+                ),
                 details=f"* Disk device configuration: {disks!r}",
             )
 
@@ -512,7 +586,7 @@ class LXDInstance(Executor):
 
         for name, _ in disks.items():
             self.lxc.config_device_remove(
-                instance_name=self.name,
+                instance_name=self.instance_name,
                 device=name,
                 project=self.project,
                 remote=self.remote,
